#include <iostream>
#include <cstdlib>
#include <cstring>
#include <cerrno>
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include <sys/socket.h>
#include <unistd.h>

#include "global.h"

using namespace std;

int main( int argc, char *const argv [] ) {

	// check and evaluate command-line argument
	if ( argc < 2 ) {
		cerr << "usage: " << argv[0] << " <transport-id>" << endl;
		return 1;
	}
	int transportID = strtol( argv[1], NULL, 10 );
	cout << "Given transport ID: " << transportID << endl;

	// create TCP socket
	int fd = socket( AF_INET, SOCK_STREAM, 0 );
	if ( fd < 0 ) {
		cerr << "socket: " << strerror(errno) << endl;
		return 1;
	}

	// bind socket to any local port
	struct sockaddr_in local;
	memset( &local, 0, sizeof(local) );
	local.sin_family = AF_INET;
	local.sin_addr.s_addr = INADDR_ANY;
	if ( bind( fd, (struct sockaddr*)&local, sizeof(local) ) < 0 ) {
		cerr << "bind: " << strerror(errno) << endl;
		return 1;
	}

	// retrieve assigned port number
	socklen_t len = sizeof(local);
	if ( getsockname( fd, (struct sockaddr*)&local, &len ) < 0 ) {
		cerr << "getsockname: " << strerror(errno) << endl;
		return 1;
	}

	cout << "Port: " << local.sin_port << endl;

	// register port number with port mapping service
	// NOTE: On Solaris, the port number that is registered here must be bound
	//       to a socket belonging to this process.
	//       On Linux, this does not seem to be necessary.
	if ( pmap_set( studentID, transportID, IPPROTO_TCP, local.sin_port ) == 0 ) {
		cerr << "pmap_set: " << strerror(errno) << endl;
		return 1;
	}

	// wait for user to finish program by pressing ENTER
	cout << "Use rpcinfo to see your registration" << endl;
	cout << "Press ENTER to finish program";
	int x = cin.get();

	// deregister from port mapping
	// NOTE: On Solaris, the port mapping is normally automatically removed
	//       from the system when the socket is removed (latest upon process
	//       termination).  On Linux, this is not the case (consistent with
	//       the previous note).  In this case, 'rpcinfo -d' can be used to
	//       remove all mappings for a certain student ID, which may have been
	//       left from crashed programs.  See 'man rpcinfo'
	if ( pmap_unset( studentID, transportID ) == 0 ) {
		cerr << "pmap_unset: " << strerror(errno) << endl;
		return 1;
	}

	// close file descriptor
	if ( close(fd) < 0 ) {
		cerr << "close: " << strerror(errno) << endl;
		return 1;
	}

	return 0;
}
