/*
 * Copyright (c) 2001-2004 Twisted Matrix Laboratories.
 * See LICENSE for details.

 * 
 */

/* portmap.c: A simple Python wrapper for pmap_set(3) and pmap_unset(3) */

#include <Python.h>
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include <sys/socket.h>
#include <errno.h>

static PyObject * portmap_set(PyObject *self, PyObject *args) {
	unsigned long program, version;
	int port;

	if (!PyArg_ParseTuple(args, "lli:set", &program, &version, &port)) return NULL;

	pmap_unset(program, version);
	pmap_set(program, version, IPPROTO_TCP, port);

	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject * portmap_unset(PyObject *self, PyObject *args) {
	unsigned long program, version;

	if (!PyArg_ParseTuple(args, "ll:unset", &program, &version)) return NULL;

	pmap_unset(program, version);

	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject * portmap_getport(PyObject *self, PyObject *args) {
	unsigned long program, version;

	if (!PyArg_ParseTuple(args, "ll:getport", &program, &version)) return NULL;

	struct sockaddr_in local;
	memset( &local, 0, sizeof(local) );
	local.sin_family = AF_INET;
	local.sin_addr.s_addr = INADDR_ANY;

	int p = pmap_getport( &local , program, version, IPPROTO_TCP );
	return Py_BuildValue("i", p);
}

static PyMethodDef PortmapMethods[] = {
	{"set", portmap_set, METH_VARARGS, "Set portmap entry."},
	{"unset", portmap_unset, METH_VARARGS, "Unset portmap entry."},
	{"getport", portmap_getport, METH_VARARGS, "Get portmap entry."},
	{NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC initportmap(void) {
	(void) Py_InitModule("portmap", PortmapMethods);
}
